#!/usr/bin/env python
# coding: utf-8

# In[8]:


def generate_round_robin_odd(num_teams):
    '''Return a list orlists where results[i][j] gives
    the opponent of team i in round j.
    Note: num_teams must be odd.
    '''
    # Make an empty list of lists.
    results = [list(range(num_teams)) for i in range(num_teams)]

    # Initialize the list of teams.
    teams = list(range(num_teams))

    # Start the rounds.
    n2 = (num_teams - 1) // 2
    for round in range(num_teams):
        for i in range(n2):
            team1 = teams[n2 - i]
            team2 = teams[n2 + i + 1]
            results[team1][round] = team2
            results[team2][round] = team1

        # Set the team with the bye.
        results[teams[0]][round] = 'bye'

        # Rotate the array.
        rotate_list(teams)

    return results


# In[9]:


def rotate_list(teams):
    '''
    Rotate the entries one position.
    Move the last item to the front of the list.
    '''
    teams.insert(0, teams.pop())


# In[10]:


def generate_round_robin_even(num_teams):
    # Generate the result for one fewer teams.
    results = generate_round_robin_odd(num_teams - 1)

    # Copy the results into a bigger array,
    # replacing the byes with the extra team.
    results2 = [list(range(num_teams - 1)) for i in range(num_teams)]
    for team in range(num_teams - 1):
        for round in range(num_teams - 1):
            if results[team][round] == 'bye':
                # Change the bye to the new team.
                results2[team][round] = num_teams - 1
                results2[num_teams - 1][round] = team
            else:
                results2[team][round] = results[team][round]

    return results2


# In[11]:


def generate_round_robin(num_teams):
    if num_teams % 2 == 0:
        return generate_round_robin_even(num_teams)
    else:
        return generate_round_robin_odd(num_teams)


# In[16]:


def generate_round_robin_names(team_names):
    # Generate the schedule.
    num_teams = len(team_names)
    schedule = generate_round_robin(num_teams)

    # Replace team numbers with names.
    name_schedule = schedule.copy()
    num_rounds = len(schedule[0])
    for team in range(num_teams):
        for round in range(num_rounds):
            if name_schedule[team][round] != 'bye':
                name_schedule[team][round] = team_names[name_schedule[team][round]]
        # Insert this team at the beginning of its list.
        name_schedule[team].insert(0, team_names[team])
    return name_schedule


# In[23]:


def print_matrix(schedule):
    for row in schedule:
        print(row)

def print_schedule(schedule):
    num_teams = len(schedule)
    num_rounds = len(schedule[0])
    for round in range(1, num_rounds):
        print(f'Round {round}')
        for team1_number in range(num_teams):
            team1 = schedule[team1_number][0]
            team2 = schedule[team1_number][round]

            # Only print if it's a bye or team 1 < team 2.
            if team2 == 'bye':
                print(f'  {team1} has a bye')
            elif team1 < team2:
                print(f'  {team1} vs. {team2}')


# In[29]:


print('Four teams:')
team_names = ['Ants', 'Bats', 'Cats', 'Dogs']
schedule = generate_round_robin_names(team_names)
#print_matrix(schedule)
print_schedule(schedule)

print()
print('Five teams:')
team_names.append('Elves')
schedule = generate_round_robin_names(team_names)
#print_matrix(schedule)
print_schedule(schedule)


# In[ ]:




